function output = assoc_model_run(params_vec, num_subs, num_lists, list_len, primacy_on)
% Implements the associative chaining model described in Solway, Murdock, Kahana (in press), 
% Memory & Cognition.
% 
% See README.

% primacy gradient is on by default.
if ~exist('primacy_on', 'var')
    primacy_on = 1;
end

output = {};

wb          = params_vec(1);
min_as      = params_vec(2);
ds          = params_vec(3);
cs          = params_vec(4);
sigma_as    = params_vec(5);
mu_bs       = params_vec(6);
sigma_bs    = params_vec(7);
null        = params_vec(8);
gamma       = params_vec(9);

if ~primacy_on
    cs = 0;
    ds = 0;
end

% maximum number of study-test trials for each list
max_trials = 10;

% all item indices. 1 refers to the "0"/start item 
all_items = 1:list_len+1;

for sno=1:num_subs
    sub_output = [];
    for lno=1:num_lists
        list_output = [];
        % Matrix of item-to-item strengths, including a "0" item.
        S = zeros(list_len+1,list_len+1);

        for tno=1:max_trials
            bs = mu_bs + sigma_bs.*randn();
            bs(bs<1) = 1;

            % Study the list.
            for ino=1:list_len
                % Sample a_s and b_s.
                mu_as = cs.*exp(-ds.*(ino-1)) + min_as;
                as    = mu_as + sigma_as.*randn();

                % Forward and backward strength increases between this item and previous items.
                forw_strength_increase = as.*(1 - S(1:ino,ino+1))'.*exp(-bs'.*[[ino:-1:1]-1]);
                forw_strength_increase(forw_strength_increase<0) = 0;
                S(1:ino,ino+1) = S(1:ino,ino+1) + forw_strength_increase';
                S(ino+1,1:ino) = S(ino+1,1:ino) + wb.*forw_strength_increase;
                
                % Keep S within bounds.
                S(S>1) = 1;
            end

            % Recall the list.
            recalled    = [1];
            last_recall = 1;  % Remember that index 1 is really the "0"th item.
            for ono=1:list_len
                % Recall an item.
                unrecalled  = all_items;
                unrecalled(recalled) = [];
                prob_recall = (S(last_recall,unrecalled).^gamma)./(sum(S(last_recall,unrecalled).^gamma) + null.^gamma);
                choose_null = 0;
                recall_idx  = 0;
                choice      = rand();
                cumulProb   = 0;
                while cumulProb < choice
                    recall_idx = recall_idx + 1;
                    if recall_idx > length(unrecalled)
                        choose_null = 1;
                        break;
                    end
                    cumulProb = cumulProb + prob_recall(recall_idx);
                end
                if choose_null
                    break;
                end
                last_recall = unrecalled(recall_idx);
                list_output = [list_output;sno lno tno last_recall-1 ono];
        
                % Mark item as recalled.
                recalled = [recalled last_recall];
            end

            % Have we learned the list?
            if length(recalled) == length(all_items) && all(recalled == all_items)
                break;
            end
        end %tno
        sub_output = [sub_output;list_output];
    end %lno
    output{sno} = sub_output';
end %sno

